import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../../core/error-logger.service';
import { Table } from '../../core/table';

import { Bill } from './bill';
import { PrintType } from './print-type';

const url = '/api/voucher';
const urlMoveTable = '/api/move-table';
const urlMoveKot = '/api/move-kot';
const urlSplitBill = '/api/split-bill';
const serviceName = 'VoucherService';

@Injectable({ providedIn: 'root' })
export class VoucherService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  get(id: string): Observable<Bill> {
    return <Observable<Bill>>(
      this.http
        .get<Bill>(`${url}/from-id/${id}`)
        .pipe(catchError(this.log.handleError(serviceName, `get id=${id}`)))
    );
  }

  getFromTable(tableId: string, voucherId: string, guestId: string): Observable<Bill> {
    let params = new HttpParams();
    if (voucherId !== null) {
      params = params.set('v', voucherId);
    }
    if (guestId !== null) {
      params = params.set('g', guestId);
    }

    return <Observable<Bill>>this.http
      .get<Bill>(`${url}/from-table/${tableId}`, { params })
      .pipe(
        catchError(
          this.log.handleError(
            serviceName,
            `getFromTable tableId=${tableId} voucherId=${voucherId} guestId=${guestId}`,
          ),
        ),
      );
  }

  save(
    voucher: Bill,
    printType: PrintType,
    guest_book_id: string,
    updateTable: boolean,
  ): Observable<boolean> {
    const options = {
      params: new HttpParams().set('p', printType).set('u', updateTable.toString()),
    };
    if (guest_book_id !== null) {
      options.params = options.params.set('g', guest_book_id);
    }
    return <Observable<boolean>>(
      this.http
        .post<boolean>(`${url}/save`, voucher, options)
        .pipe(catchError(this.log.handleError(serviceName, 'save')))
    );
  }

  update(
    voucher: Bill,
    printType: PrintType,
    guest_book_id: string,
    updateTable: boolean,
  ): Observable<boolean> {
    const options = {
      params: new HttpParams().set('p', printType).set('u', updateTable.toString()),
    };
    if (guest_book_id !== null) {
      options.params = options.params.set('g', guest_book_id);
    }
    return <Observable<boolean>>(
      this.http
        .put<boolean>(`${url}/update/${voucher.id}`, voucher, options)
        .pipe(catchError(this.log.handleError(serviceName, 'update')))
    );
  }

  change(
    voucher: Bill,
    printType: PrintType,
    guest_book_id: string,
    updateTable: boolean,
  ): Observable<boolean> {
    const options = { params: new HttpParams().set('u', updateTable.toString()) };
    if (guest_book_id !== null) {
      options.params = options.params.set('g', guest_book_id);
    }
    return <Observable<boolean>>(
      this.http
        .put<boolean>(`${url}/change/${voucher.id}`, voucher, options)
        .pipe(catchError(this.log.handleError(serviceName, 'change')))
    );
  }

  saveOrUpdate(
    voucher: Bill,
    printType: PrintType,
    guest_book_id: string,
    updateTable: boolean,
  ): Observable<boolean> {
    if (!voucher.id) {
      return this.save(voucher, printType, guest_book_id, updateTable);
    }
    if (voucher.voucherType === PrintType.Kot) {
      return this.update(voucher, printType, guest_book_id, updateTable);
    }
    return this.change(voucher, printType, guest_book_id, updateTable);
  }

  receivePayment(
    id: string,
    amounts: { id: number; name: string; amount: number }[],
    name: string,
    updateTable: boolean,
  ): Observable<boolean> {
    const options = { params: new HttpParams().set('u', updateTable.toString()) };
    return <Observable<boolean>>(
      this.http
        .post<boolean>(`${url}/receive-payment/${id}`, { name, amounts }, options)
        .pipe(catchError(this.log.handleError(serviceName, 'receivePayment')))
    );
  }

  voidBill(id: string, reason: string, updateTable: boolean): Observable<boolean> {
    const options = {
      params: new HttpParams().set('reason', reason).set('u', updateTable.toString()),
    };
    return <Observable<boolean>>(
      this.http
        .post<boolean>(`${url}/void-bill/${id}`, {}, options)
        .pipe(catchError(this.log.handleError(serviceName, 'voidBill')))
    );
  }

  moveTable(id: string, table: Table): Observable<boolean> {
    return <Observable<boolean>>this.http
      .post<boolean>(`${urlMoveTable}/move`, {
        voucherId: id,
        tableId: table.id,
      })
      .pipe(catchError(this.log.handleError(serviceName, 'moveTable')));
  }

  mergeTable(id: string, table: Table): Observable<boolean> {
    return <Observable<boolean>>this.http
      .post<boolean>(`${urlMoveTable}/merge`, {
        voucherId: id,
        tableId: table.id,
        newVoucherId: table.voucherId,
      })
      .pipe(catchError(this.log.handleError(serviceName, 'mergeTable')));
  }

  moveKotToNewTable(id: string, kotId: string, table: Table): Observable<boolean> {
    return <Observable<boolean>>this.http
      .post<boolean>(`${urlMoveKot}/move`, {
        voucherId: id,
        kotId,
        tableId: table.id,
      })
      .pipe(catchError(this.log.handleError(serviceName, 'moveKotToNewTable')));
  }

  mergeKotWithOldBill(id: string, kotId: string, table: Table): Observable<boolean> {
    return <Observable<boolean>>this.http
      .post<boolean>(`${urlMoveKot}/merge`, {
        voucherId: id,
        kotId,
        tableId: table.id,
        newVoucherId: table.voucherId,
      })
      .pipe(catchError(this.log.handleError(serviceName, 'mergeKotWithOldBill')));
  }

  splitBill(id: string, inventoriesToMove: string[], table: Table) {
    const options = { params: new HttpParams().set('u', 'true') };
    return <Observable<boolean>>this.http
      .post<boolean>(
        `${urlSplitBill}/${id}`,
        {
          inventories: inventoriesToMove,
          tableId: table.id,
        },
        options,
      )
      .pipe(catchError(this.log.handleError(serviceName, 'splitBill')));
  }
}
